/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "mptkcodec.h"

int main( int argc, char **argv )
{
	/* Declarations */
	book_coder *coder;
	MP_Book_c *book;
	unsigned int numBytes, n;
	long unsigned int numSamples;
	FILE *fid;

	/* Check number of arguments */
	if (ENCODE_OUT_WAV) {
		if ( argc!=6 ) {
			printf("Needs 5 arguments: [bookfile] [codedfile] [bitrate in bps] [mode] [wavfile]\n");
			return 1;
		}
	} else {
		if ( argc!=5 ) {
			printf("Needs 4 arguments: [bookfile] [codedfile] [bitrate in bps] [mode]\n");
			return 1;
		}
	}

	/* Load book */
	book =  MP_Book_c::init();
	fid = fopen(argv[1],"r");
	book->load( fid );
	fread(&numSamples,sizeof(long unsigned int),1,fid);
	fclose(fid);

	/* Open output file */
	fid = fopen(argv[2],"w");

	/* Initialize coder */
	coder = new book_coder(book,numSamples,atoi(argv[4]));
	coder->init_bitplane();

	/* Init synthesized signal */
	if ( ENCODE_OUT_WAV ) {
		coder->init_signal();
	}

	/* Write Header */
	printf("Write header...");
	numBytes = coder->write_header( fid );
	printf("%u bytes\n",numBytes);

	/* Write data */
	for ( n=0 ; n<coder->numSlots ; n++ ) {
		printf("Write slot %u/%u...",n+1,coder->numSlots);
		coder->load_slot( coder->firstNonZeroSlot+n );
		coder->interleave_slot();
		numBytes = coder->code_slot( atoi(argv[3]) );
		if ( ENCODE_OUT_WAV ) coder->synth_coded_slot( coder->firstNonZeroSlot+n );
		fwrite(&numBytes,sizeof(unsigned int),1,fid );
		coder->write_slot( fid );
		printf("%u bytes\n",numBytes);
	}

	/* Save decoded wavfile */
	if (ENCODE_OUT_WAV) {
		if (EDGES_ZEROS) {
			MP_Signal_c *signal2 = MP_Signal_c::init( coder->signal->numChans, coder->signal->numSamples-2*coder->signal->sampleRate, coder->signal->sampleRate );
			memcpy( signal2->storage, coder->signal->storage+coder->signal->sampleRate, signal2->numSamples*sizeof(MP_Sample_t) );
			signal2->wavwrite( argv[5] );
			delete( signal2 );
		} else {
			coder->signal->wavwrite( argv[5] );
		}
	}

	/* Clean */
	delete( book );
	delete( coder );
	fclose( fid );
	fftw_cleanup();

	return 0;
}
